import React, { useEffect } from 'react';
import { Card, Row, Col, Table, message } from 'antd';
import _ from 'lodash';
import { C, Icon } from 'xadmin-ui'
import './styles/Home.css'
import { api, app } from 'xadmin'
import echarts from 'echarts'
import moment from 'moment'
import elementResizeEvent from 'element-resize-event'
import ServiceCardSvg from './icon/ServiceCard.svg'
import ModularCardSvg from './icon/ModularCard.svg'

const columns = type => {
  return [
    {
      title: type == 'server' ? '服务名称' : '模块名称',
      dataIndex: 'name',
      key: 'name',
    },
    {
      title: '更新时间',
      dataIndex: 'time',
      key: 'time',
      render: (text, record) => {
        return <span>{record.time ? moment(record.time).format('YYYY-MM-DD HH:mm:ss') : null}</span>
      }
    },
    {
      title: '更新信息',
      dataIndex: 'msg',
      key: 'msg'
    },
    {
      title: '更新版本',
      dataIndex: 'version',
      key: 'version',
      render: (version, values) => {
        let result = ''
        if (values.name && values.name.split('/')?.length > 5) {
          if (values.name.split('/')[4].startsWith('v')) result = values.name.split('/')[4]
        }
        return <span>{version === '' ? result : version}</span>
      }
    }
  ]
}

const HomeCard = ({ colData }) => {
  const cols = []
  colData.forEach((item, i) => {
    let router = ['/app/model/Service/list', '/app/model/Modular/list']
    cols.push(
      <Col className="ant-col-home" key={i} span={12} >
        <Card hoverable bodyStyle={{ padding: 0 }} onClick={() => app.go(router[i])} >
          <Row>
            <Col span={10} className="home-head">
              <div className="homeCard" style={{ width: 206, height: 206 }}>
                <img style={{ width: 60 }} src={item.icon}></img>
              </div>
            </Col>
            <Col span={14} className="home-head">
              <div>
                <Row><div className="home-title">{item.title}</div></Row>
                <Row><span className="span-num">{item.num}</span></Row>
              </div>
            </Col>
          </Row>
        </Card>
      </Col>
    )
  })

  return (
    <div className="homeBox">
      <Row className="ant-min-row" gutter={[48, 8]}>{cols}</Row>
    </div>
  )
}

const HomeStatus = ({ serviceArr, width }) => {
  const runCount = serviceArr.filter(v => v.state === 'running').length
  const exitedCount = serviceArr.filter(v => v.state !== 'running').length

  const ref = React.useRef(null)

  React.useEffect(() => {
    let myChart = echarts.init(document.getElementById('main-echart'))
    const option = {
      tooltip: {
        trigger: 'item',
        formatter: '{a} <br/>{b} : {c} ({d}%) ',
        textStyle: { align: 'center' }
      },
      color: ['#603BE7', '#FFD982',],
      legend: {
        orient: 'vertical',
        left: 'left',
        data: ["运行", "停止"]
      },
      series: [
        {
          name: '服务状态',
          type: 'pie',
          radius: '55%',
          center: ['50%', '60%'],
          data: [
            { value: runCount || 0, name: '运行' },
            { value: exitedCount || 0, name: '停止' },
          ],
          itemStyle: {
            emphasis: {
              shadowBlur: 10,
              shadowOffsetX: 0,
              shadowColor: 'rgba(0, 0, 0, 0.5)'
            }
          }
        }
      ]
    }
    myChart.setOption(option)

    elementResizeEvent(ref.current, () => {
      myChart.resize()
    })
  })

  return (
    <Col span={24}>
      <Card hoverable bodyStyle={{ padding: 10 }} >
        <div ref={ref} id="main-echart" style={{ width: '100%', height: 220, fontSize: '12px' }}></div>
      </Card>
    </Col>
  )
}

const HomeContent = ({ url }) => {

  const [serviceArr, setServiceArr] = React.useState([])
  const [colData, setColData] = React.useState([])
  const [log, setLog] = React.useState([])

  const initData = [
    {
      icon: ServiceCardSvg,
      title: '服务总数',
      bgColor: '#FFE5E5',
      num: null,
      core: `api/${url}/service`,
      id: 'service'
    },
    {
      icon: ModularCardSvg,
      title: '模块总数',
      bgColor: '#FFE5E5',
      num: null,
      core: 'api/front',
      id: 'front'
    }
  ]

  useEffect(() => {
    console.log('运维版本号：0826.1030')
    let promiseArray = []
    initData.forEach(item => {
      promiseArray.push(new Promise((resolve, reject) => {
        let queryStr = encodeURIComponent(JSON.stringify({ withoutBody: true }))
        api({ name: `${item.core}` })
          .fetch('?query=' + queryStr, {})
          .then(res => {
            item.title === '服务总数' && setServiceArr(res.json)
            let successVal = res.headers.count || res.json.length
            resolve({ successVal, status: 'success' })
          }).catch(err => {
            resolve({ err, status: 'fail' })
          })
      }))
    })

    Promise.all(promiseArray).then(res => {
      let resData = res && res.map((item, index) => {
        if (item.status == 'success') {
          return { ...initData[index], num: item.successVal && Number(item.successVal) || '-' }
        } else if (item.status == 'fail') {
          return { ...initData[index], num: '-' }
        }
      })
      setColData(resData)
    })

    api({ name: `api/log` })
      .fetch('', {})
      .then(({ json }) => {
        if (json && json.length) {
          setLog(json)
        }
      })
  }, [])

  return (
    <div style={{ overflow: 'hidden', padding: '0 2px' }}>
      <Row>
        <Col span={12}>
          <div className="home-module" >
            <div></div>
            <span>统计</span>
          </div>
          <div className="home-module-card">
            <HomeCard colData={colData}></HomeCard>
          </div>
        </Col>
        <Col span={12}>
          <div className="home-module">
            <div></div>
            <span>服务状态</span>
          </div>
          <div className="home-module-card">
            <HomeStatus serviceArr={serviceArr} />
          </div>
        </Col>
      </Row>
      <Row>
        <Col span={12}>
          <div style={{ display: 'flex' }}>
            <div className="home-module home-module-left">
              <div></div>
              <span>服务更新日志</span>
            </div>
          </div>
          <div className="home-module-card">
            <Card bodyStyle={{ padding: 0 }} bordered={false}>
              <Table
                columns={columns('server')}
                dataSource={log.filter(v => v.group == 'server' || v.group == 'driver')}
                bordered
                pagination={false}
                locale={{ emptyText: "暂无数据" }}
                scroll={{ y: document.body.clientHeight-480 }}
              />
            </Card>
          </div>
        </Col>
        <Col span={12}>
          <div style={{ display: 'flex' }}>
            <div className="home-module home-module-left">
              <div></div>
              <span>模块更新日志</span>
            </div>
          </div>
          <div className="home-module-card">
            <Card bodyStyle={{ padding: 0 }} bordered={false}>
              <Table 
                columns={columns('front')} 
                dataSource={log.filter(v => v.group == 'front')} 
                bordered 
                pagination={false} 
                locale={{ emptyText: "暂无数据" }} 
                scroll={{ y: document.body.clientHeight-480 }} 
              />
            </Card>
          </div>
        </Col>
      </Row>
    </div>
  )
}

export default HomeContent