import React from 'react'
import { api, app } from 'xadmin'
import { Page } from 'xadmin-ui';
import echarts from 'echarts'
import moment from 'moment'
import _ from 'lodash'
import { Card, Row, Col, Select } from 'antd';
import { AreaChartOutlined, ArrowLeftOutlined } from '@ant-design/icons';
const { Option } = Select;

const optionInit = {
  memory: {
    tooltip: {
      trigger: 'axis',
      formatter: function (datas) {
        var res = datas[0].axisValue + '<br/>'
        for (var i = 0, length = datas.length; i < length; i++) {
           res += `<div style="width: 14px; height: 14px; background-color: ${datas[i].color}; display: inline-block"></div> `
               + datas[i].seriesName + '：' 
               + datas[i].data.toFixed(4) + 'MB' + '<br/>'
        }
        return res
      },
      axisPointer: {
        type: 'cross',
        label: {
          backgroundColor: '#6a7985'
        }
      }
    },
    legend: {
      data: ['内存', '缓存']
    },
    grid: {
      left: '3%',
      right: '4%',
      bottom: '3%',
      containLabel: true
    },
    xAxis: [
      {
        type: 'category',
        axisLabel: {
          rotate:40  
        },
        boundaryGap: false
      }
    ],
    yAxis: [
      {
        type: 'value',
        axisLabel: {
          formatter:'{value} MB'
        }
      }
    ],
    series: [
      {
        name: '内存',
        type: 'line',
        color: '#70a2d6',
        areaStyle: {
          color: '#70a2d6'
        },
        emphasis: {
          focus: 'series'
        }
      },
      {
        name: '缓存',
        type: 'line',
        color: '#e69087',
        areaStyle: {
          color: '#e69087'
        },
        emphasis: {
          focus: 'series'
        }
      }
    ]
  },
  cpu: {
    tooltip: {
      trigger: 'axis',
      formatter: function (datas) {
        var res = datas[0].axisValue + '<br/>'
        for (var i = 0, length = datas.length; i < length; i++) {
           res += `<div style="width: 14px; height: 14px; background-color: ${datas[i].color}; display: inline-block"></div> `
               + datas[i].seriesName + '：'
               + datas[i].data.toFixed(3) + '%' + '<br/>'
        }
        return res
      },
      axisPointer: {
        type: 'cross',
        label: {
          backgroundColor: '#6a7985'
        }
      }
    },
    legend: {
      data: ['CPU']
    },
    grid: {
      left: '3%',
      right: '4%',
      bottom: '3%',
      containLabel: true
    },
    xAxis: [
      {
        type: 'category',
        axisLabel: {
          rotate:40  
        },
        boundaryGap: false
      }
    ],
    yAxis: [
      {
        type: 'value',
        axisLabel: {
          formatter:'{value} %'
        }
      }
    ],
    series: [
      {
        name: 'CPU',
        type: 'line',
        color: '#70a2d6',
        areaStyle: {
          color: '#70a2d6'
        },
        emphasis: {
          focus: 'series'
        }
      }
    ]
  },
  network: {
    tooltip: {
      trigger: 'axis',
      formatter: function (datas) {
        var res = datas[0].axisValue + '<br/>'
        for (var i = 0, length = datas.length; i < length; i++) {
           res += `<div style="width: 14px; height: 14px; background-color: ${datas[i].color}; display: inline-block"></div> `
               + datas[i].seriesName + '：'
               + datas[i].data.toFixed(1) + 'MB' + '<br/>'
        }
        return res
      },
      axisPointer: {
        type: 'cross',
        label: {
          backgroundColor: '#6a7985'
        }
      }
    },
    legend: {
      data: ['RX on eth0', 'TX on eth0']
    },
    grid: {
      left: '3%',
      right: '4%',
      bottom: '3%',
      containLabel: true
    },
    xAxis: [
      {
        type: 'category',
        axisLabel: {
          rotate:40  
        },
        boundaryGap: false
      }
    ],
    yAxis: [
      {
        type: 'value',
        axisLabel: {
          formatter:'{value} MB'
        }
      }
    ],
    series: [
      {
        name: 'RX on eth0',
        type: 'line',
        color: '#70a2d6',
        emphasis: {
          focus: 'series'
        }
      },
      {
        name: 'TX on eth0',
        type: 'line',
        color: '#e69087',
        emphasis: {
          focus: 'series'
        }
      }
    ]
  }
}

const StatsPage = props => {
  const { location: { query } } = props
  const podName = query && query.podName
  const [echartData, setEchartData] = React.useState({
    timeList: [],
    memory: {
      mList: [],
      cList: []
    },
    cpu: [],
    network: {
      rxList: [],
      txList: []
    }
  })
  const [refreshTime, setRefreshTime] = React.useState(5)
  const refreshInterval = React.useRef(null)

  React.useEffect(() => {
    initChart()
    getStats()
  }, [])

  React.useEffect(() => {
    initChart()
  }, [echartData])

  React.useEffect(() => {
    if (refreshInterval.current) {
      clearInterval(refreshInterval.current)
    }
    refreshInterval.current = setInterval(() => {
      getStats()
    }, refreshTime * 1000)
    return () => {
      clearInterval(refreshInterval.current)
      refreshInterval.current = null
    }
  }, [refreshTime])

  // 初始化
  const initChart = () => {
    let memoryEchart = echarts.init(document.getElementById('memory-echart'))
    memoryEchart.setOption({
      ...optionInit.memory,
      xAxis: [
        {
          ...optionInit.memory.xAxis[0],
          data: echartData.timeList
        }
      ],
      series: [
        {
          ...optionInit.memory.series[0],
          data: echartData.memory.mList
        },
        {
          ...optionInit.memory.series[1],
          data: echartData.memory.cList
        }
      ]
    })
    let cpuEchart = echarts.init(document.getElementById('cpu-echart'))
    cpuEchart.setOption({
      ...optionInit.cpu,
      xAxis: [
        {
          ...optionInit.cpu.xAxis[0],
          data: echartData.timeList
        }
      ],
      series: [
        {
          ...optionInit.cpu.series[0],
          data: echartData.cpu
        }
      ]
    })
    let networkEchart = echarts.init(document.getElementById('network-echart'))
    networkEchart.setOption({
      ...optionInit.network,
      xAxis: [
        {
          ...optionInit.network.xAxis[0],
          data: echartData.timeList
        }
      ],
      series: [
        {
          ...optionInit.network.series[0],
          data: echartData.network.rxList
        },
        {
          ...optionInit.network.series[1],
          data: echartData.network.txList
        }
      ]
    })
  }

  // 获取数据
  const getStats = () => {
    api({ name: 'api' }).fetch(`/docker/containers/${podName}/stats`, {})
      .then(({ status, body }) => {
        if (status === 200 && body) {
          let obj = JSON.parse(body)
          let memoryData = obj?.memory_stats?.usage/1024/1024
          let cacheData = obj?.memory_stats?.stats?.cache/1024/1024
          let time = moment().format('HH:mm:ss')
          let cpuData = obj?.cpu_stats?.cpu_usage?.total_usage/obj.cpu_stats.system_cpu_usage * 100 * 100
          let rxData = obj?.networks?.eth0?.rx_bytes/1024/1024
          let txData = obj?.networks?.eth0?.tx_bytes/1024/1024
          setEchartData(before => {
            let result = _.cloneDeep(before)
            result = {
              timeList: [...before.timeList, time],
              memory: {
                mList: [...before.memory.mList, memoryData],
                cList: [...before.memory.cList, cacheData]
              },
              cpu: [...before.cpu, cpuData],
              network: {
                rxList: [...before.network.rxList, rxData],
                txList: [...before.network.txList, txData]
              }
            }
            return result
          })
        }
      })
  }

  return (
    <Page title={<span><ArrowLeftOutlined onClick={() => app.go('/app/model/Service/list')} /> 图表</span>}>
      <Row>
        <Col style={{ padding: '0px 15px' }} span={24}>
          <Card bodyStyle={{ padding: 20 }} >
            <span>刷新时间</span>
            <Select defaultValue={5} style={{ width: 120, marginLeft: 20 }} onChange={value => setRefreshTime(value)}>
              <Option value={1}>1s</Option>
              <Option value={3}>3s</Option>
              <Option value={5}>5s</Option>
              <Option value={10}>10s</Option>
              <Option value={30}>30s</Option>
              <Option value={60}>60s</Option>
            </Select>
          </Card>
        </Col>
      </Row>
      <Row>
        <Col style={{ padding: 15 }} span={8}>
          <Card title={<><AreaChartOutlined /> 内存使用情况</>} hoverable bodyStyle={{ padding: 20 }} >
            <div id="memory-echart" style={{ width: '90%', height: 240, fontSize: '12px' }}></div>
          </Card>
        </Col>
        <Col style={{ padding: 15 }} span={8}>
          <Card title={<><AreaChartOutlined /> CPU使用情况</>} hoverable bodyStyle={{ padding: 20 }} >
            <div id="cpu-echart" style={{ width: '90%', height: 240, fontSize: '12px' }}></div>
          </Card>
        </Col>
        <Col style={{ padding: 15 }} span={8}>
          <Card title={<><AreaChartOutlined /> 网络使用情况</>} hoverable bodyStyle={{ padding: 20 }} >
            <div id="network-echart" style={{ width: '90%', height: 240, fontSize: '12px' }}></div>
          </Card>
        </Col>
      </Row>
    </Page>
  )
}

export default StatsPage